package org.processmining.plugins.simplelogoperations;

import java.util.Date;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeMapImpl;
import org.deckfour.xes.model.impl.XEventImpl;
import org.deckfour.xes.model.impl.XLogImpl;
import org.deckfour.xes.model.impl.XTraceImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;

public class AddEvents {
	@Plugin(name = "Add Artificial Events", 
			parameterLabels = { "Log" }, 
			returnLabels = { "Altered log" }, 
			returnTypes = { XLog.class }, 
			userAccessible = true, 
			help = "Adds an artificial start and/or end task to every trace in the log file")
	@UITopiaVariant(affiliation = "Ghent University", 
	        author = "Jan Claes", 
	        email = "jan.claes@ugent.be", 
	        website = "http://www.janclaes.info")
	public XLog addEvents(UIPluginContext context, XLog oldLog) {
		AddEventsPanel parameters = new AddEventsPanel();
		InteractionResult interActionResult = context.showConfiguration("Add artificial events", parameters);
		if (interActionResult.equals(InteractionResult.CANCEL)) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		context.getFutureResult(0).setLabel(XConceptExtension.instance().extractName(oldLog));

		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(oldLog.size());
		context.getProgress().setIndeterminate(false);
		context.getProgress().setValue(0);

		XAttributeMap logattlist = XLogFunctions.copyAttMap(oldLog.getAttributes());
		XLog newLog = new XLogImpl(logattlist);
		for (int i = 0; i < oldLog.size(); i++) {
			XTrace oldTrace = oldLog.get(i);
			XTrace newTrace = new XTraceImpl(XLogFunctions.copyAttMap(oldTrace.getAttributes()));
			if (parameters.isAddStartTask()) {
				Date time = new Date();
				try {
					time = XLogFunctions.getTime(oldTrace.get(0));
					time.setTime(time.getTime() - 1);
				} catch (Exception ex) {
				}
				newTrace.add(makeEvent(parameters.getStartTaskName(), time));
			}
			for (int j = 0; j < oldTrace.size(); j++) {
				XEvent oldEvent = oldTrace.get(j);
				XEvent newEvent = new XEventImpl(XLogFunctions.copyAttMap(oldEvent.getAttributes()));
				newTrace.add(newEvent);
			}
			if (parameters.isAddStopTask()) {
				Date time = new Date();
				try {
					time = XLogFunctions.getTime(oldTrace.get(oldTrace.size() - 1));
					time.setTime(time.getTime() + 1);
				} catch (Exception ex) {
				}
				newTrace.add(makeEvent(parameters.getStopTaskName(), time));
			}
			newLog.add(newTrace);
			context.getProgress().inc();
		}
		return newLog;
	}

	private XEvent makeEvent(String name, Date time) {
		XAttributeMap attMap = new XAttributeMapImpl();
		XLogFunctions.putLiteral(attMap, "concept:name", name);
		XLogFunctions.putLiteral(attMap, "lifecycle:transition", "complete");
		XLogFunctions.putLiteral(attMap, "org:resource", "artificial");
		XLogFunctions.putTimestamp(attMap, "time:timestamp", time);
		XEvent newEvent = new XEventImpl(attMap);
		return newEvent;
	}
}
