package org.processmining.plugins.simplelogoperations;

import java.util.Date;
import java.util.Iterator;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeMapImpl;
import org.deckfour.xes.model.impl.XAttributeTimestampImpl;
import org.deckfour.xes.model.impl.XEventImpl;
import org.deckfour.xes.model.impl.XLogImpl;
import org.deckfour.xes.model.impl.XTraceImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;

public class RelativeTimestamps {
	@Plugin(name = "Make timestamps relative",
			parameterLabels = { "Log" },
			returnLabels = { "Altered log" },
			returnTypes = { XLog.class },
			userAccessible = true,
			help = "Make timestamps relative to a fixed start and/or end time")
	@UITopiaVariant(affiliation = "Ghent University",
			author = "Jan Claes",
			email = "jan.claes@ugent.be",
			website = "http://www.janclaes.info")
	public XLog addEvents(UIPluginContext context, XLog oldLog) {
		RelativeTimestampsPanel parameters = new RelativeTimestampsPanel();
		InteractionResult interActionResult = context.showConfiguration("Make timestamps relative", parameters);
		if (interActionResult.equals(InteractionResult.CANCEL)) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		context.getFutureResult(0).setLabel(XConceptExtension.instance().extractName(oldLog));

		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(oldLog.size());
		context.getProgress().setIndeterminate(false);
		context.getProgress().setValue(0);

		XAttributeMap logattlist = copyAttMap(oldLog.getAttributes());
		XLog newLog = new XLogImpl(logattlist);
		for (int i = 0; i < oldLog.size(); i++) {
			XTrace oldTrace = oldLog.get(i);
			XTrace newTrace = new XTraceImpl(copyAttMap(oldTrace.getAttributes()));
			if (parameters.isStartTimeSet() && parameters.isEndTimeSet()) {
				long newStartTime = parameters.getStartTime().getTime();
				long newEndTime = parameters.getEndTime().getTime();
				long currStartTime = newStartTime; 
				long currEndTime = newEndTime;
				try {
					currStartTime = ((XAttributeTimestampImpl) oldTrace.get(0).getAttributes().get("time:timestamp")).getValue().getTime();
					currEndTime = ((XAttributeTimestampImpl) oldTrace.get(oldTrace.size()-1).getAttributes().get("time:timestamp")).getValue().getTime();
				} catch (Exception ex) {
					System.out.println(ex.getMessage());
				}
				double factor = 1.0;
				if (currEndTime != currStartTime)
					factor = (1.0*newEndTime - newStartTime) / (currEndTime - currStartTime);
				for (int j = 0; j < oldTrace.size(); j++) {
					XEvent oldEvent = oldTrace.get(j);
					XEvent newEvent = new XEventImpl(copyAttMap(oldEvent.getAttributes()));
					if (oldEvent.getAttributes().containsKey("time:timestamp")) {
						long currTime = ((XAttributeTimestampImpl) oldEvent.getAttributes().get("time:timestamp")).getValue().getTime();
						Date newTime = new Date();
						if (currStartTime == currEndTime)
							newTime.setTime(newStartTime);
						else
							newTime.setTime(Math.round(newStartTime + ((currTime - currStartTime) * factor)));
						newEvent.getAttributes().put("time:timestamp", new XAttributeTimestampImpl("time:timestamp", newTime));
					}
					newTrace.add(newEvent);
				}
			}
			else if (parameters.isStartTimeSet()) {
				long newStartTime = parameters.getStartTime().getTime();
				long currStartTime = newStartTime; 
				try {
					currStartTime = ((XAttributeTimestampImpl) oldTrace.get(0).getAttributes().get("time:timestamp")).getValue().getTime();
				} catch (Exception ex) {
				}
				long factor = newStartTime - currStartTime;
				for (int j = 0; j < oldTrace.size(); j++) {
					XEvent oldEvent = oldTrace.get(j);
					XEvent newEvent = new XEventImpl(copyAttMap(oldEvent.getAttributes()));
					if (oldEvent.getAttributes().containsKey("time:timestamp")) {
						long currTime = ((XAttributeTimestampImpl) oldEvent.getAttributes().get("time:timestamp")).getValue().getTime();
						Date newTime = new Date();
						newTime.setTime(currTime + factor);
						newEvent.getAttributes().put("time:timestamp", new XAttributeTimestampImpl("time:timestamp", newTime));
					}
					newTrace.add(newEvent);
				}
			}
			else if (parameters.isEndTimeSet()) {
				long newEndTime = parameters.getEndTime().getTime();
				long currEndTime = newEndTime; 
				try {
					currEndTime = ((XAttributeTimestampImpl) oldTrace.get(oldTrace.size()-1).getAttributes().get("time:timestamp")).getValue().getTime();
				} catch (Exception ex) {
				}
				long factor = newEndTime - currEndTime;
				for (int j = 0; j < oldTrace.size(); j++) {
					XEvent oldEvent = oldTrace.get(j);
					XEvent newEvent = new XEventImpl(copyAttMap(oldEvent.getAttributes()));
					if (oldEvent.getAttributes().containsKey("time:timestamp")) {
						long currTime = ((XAttributeTimestampImpl) oldEvent.getAttributes().get("time:timestamp")).getValue().getTime();
						Date newTime = new Date();
						newTime.setTime(currTime + factor);
						newEvent.getAttributes().put("time:timestamp", new XAttributeTimestampImpl("time:timestamp", newTime));
					}
					newTrace.add(newEvent);
				}
			}
			newLog.add(newTrace);
			context.getProgress().inc();
		}
		return newLog;
	}
	private static XAttributeMap copyAttMap(XAttributeMap srcAttMap) {
		XAttributeMap destAttMap = new XAttributeMapImpl();
		Iterator<XAttribute> attit = srcAttMap.values().iterator();
		while (attit.hasNext()) {
			XAttribute att = attit.next();
			String key = att.getKey();
			att = (XAttribute) att.clone();

			if (!key.equals("time:timestamp"))
				destAttMap.put(key, att);
		}
		return destAttMap;
	}
}
