package org.processmining.plugins.simplelogoperations;

import java.awt.Color;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class RelativeTimestampsPanel extends JPanel {
	private static final long serialVersionUID = 0;

	private JPanel panel;

	private JLabel title;
	private JLabel lStart, lStop;
	private JCheckBox cStart, cStop;
	private JTextField tStart, tStop;

	public RelativeTimestampsPanel() {
		this.init();
	}

	private void init() {
		SlickerFactory factory = SlickerFactory.instance();
		SlickerDecorator decorator = SlickerDecorator.instance();
		panel = factory.createRoundedPanel(15, Color.gray);

		this.title = factory.createLabel("Make timestamps relative");
		this.title.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		this.title.setForeground(new Color(40, 40, 40));

		this.lStart = factory.createLabel("Start all traces at (DD.MM.YYYY hh:mm)");
		this.lStart.setForeground(new Color(40, 40, 40));
		this.lStop = factory.createLabel("End all traces at (DD.MM.YYYY hh:mm)");
		this.lStop.setForeground(new Color(40, 40, 40));

		this.cStart = new JCheckBox();
		this.cStart.setBackground(Color.GRAY);
		this.cStart.setSelected(true);
		decorator.decorate(this.cStart);
		this.cStop = new JCheckBox();
		this.cStop.setBackground(Color.GRAY);
		this.cStop.setSelected(true);
		decorator.decorate(this.cStop);

		this.tStart = new JTextField(50);
		Calendar c = new GregorianCalendar();
		c.set(2011,0,1,0,0);
		this.tStart.setText((new SimpleDateFormat("dd.MM.yyyy HH:mm")).format(c.getTime()));
		this.tStart.setForeground(new Color(40, 40, 40));
		this.tStop = new JTextField(50);
		this.tStop.setText((new SimpleDateFormat("dd.MM.yyyy HH:mm")).format(new Date()));
		this.tStop.setForeground(new Color(40, 40, 40));

		this.panel.setLayout(null);
		this.panel.add(this.title);
		this.panel.add(this.lStart);
		this.panel.add(this.cStart);
		this.panel.add(this.tStart);
		this.panel.add(this.lStop);
		this.panel.add(this.cStop);
		this.panel.add(this.tStop);

		this.panel.setBounds(0, 0, 700, 200);
		this.title.setBounds(10, 10, 200, 30);
		this.cStart.setBounds(20, 50, 20, 20);
		this.lStart.setBounds(50, 50, 230, 20);
		this.tStart.setBounds(280, 50, 200, 20);
		this.cStop.setBounds(20, 80, 20, 20);
		this.lStop.setBounds(50, 80, 230, 20);
		this.tStop.setBounds(280, 80, 200, 20);

		this.setLayout(null);
		this.add(panel);
		this.validate();
		this.repaint();
	}

	public boolean isStartTimeSet() {
		return cStart.isSelected();
	}

	public Date getStartTime() {
		try {
			return (new SimpleDateFormat("dd.MM.yyyy HH:mm")).parse(tStart.getText());
		}
		catch (ParseException pe) {
			return new Date();
		}
	}

	public boolean isEndTimeSet() {
		return cStop.isSelected();
	}

	public Date getEndTime() {
		try {
			return (new SimpleDateFormat("dd.MM.yyyy HH:mm")).parse(tStop.getText());
		}
		catch (ParseException pe) {
			return new Date();
		}
	}
}
