package org.processmining.plugins.simplelogoperations;

import java.util.Iterator;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.deckfour.xes.model.impl.XAttributeMapImpl;
import org.deckfour.xes.model.impl.XEventImpl;
import org.deckfour.xes.model.impl.XLogImpl;
import org.deckfour.xes.model.impl.XTraceImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;

public class RenameMergeEvents {
	@Plugin(name = "Rename/Merge Events",
			parameterLabels = { "Log" },
			returnLabels = { "Altered log" },
			returnTypes = { XLog.class },
			userAccessible = true,
			help = "Renames or merges events in the log")
	@UITopiaVariant(affiliation = "Ghent University",
			author = "Jan Claes",
			email = "jan.claes@ugent.be",
			website = "http://www.janclaes.info")
	public XLog renameMergeEvents(UIPluginContext context, XLog oldLog) {
		RenameMergeEventsPanel parameters = new RenameMergeEventsPanel();
		InteractionResult interActionResult = context.showConfiguration("Rename/Merge Events", parameters);
		if (interActionResult.equals(InteractionResult.CANCEL)) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		RenameMergeEventSettings settings = parameters.getSettings();

		context.getFutureResult(0).setLabel(XConceptExtension.instance().extractName(oldLog));

		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(oldLog.size());
		context.getProgress().setIndeterminate(false);
		context.getProgress().setValue(0);

		XAttributeMap logattlist = copyAttMap(oldLog.getAttributes(), settings);
		XLog newLog = new XLogImpl(logattlist);
		for (int i = 0; i < oldLog.size(); i++) {
			XTrace oldTrace = oldLog.get(i);
			XTrace newTrace = new XTraceImpl(copyAttMap(oldTrace.getAttributes(), settings));
			for (int j = 0; j < oldTrace.size(); j++) {
				XEvent oldEvent = oldTrace.get(j);
				for (int k = 0; k < settings.numMergeSettings(); k++) {
					MergeSetting msetting = settings.getMergeSetting(k);
					if (getName(oldEvent).equals(msetting.getFrom1()) && j + 1 < oldTrace.size()
							&& getName(oldTrace.get(j + 1)).equals(msetting.getFrom2())) {
						//if second events is of type "complete", use second event as model, otherwise use first
						XEvent modelEvent = isCompleteEvent(oldTrace.get(j + 1)) ? oldTrace.get(j + 1) : oldEvent;
						//copy modelevent in 'oldevent' variable
						oldEvent = new XEventImpl(copyAttMap(modelEvent.getAttributes(), settings));
						//change name for the 'merged event'
						setName(oldEvent, msetting.getInto());
						//set j to skip handling next event
						j++;
						//do not break, but continue with next mergesetting...
					}
				}
				XEvent newEvent = new XEventImpl(copyAttMap(oldEvent.getAttributes(), settings));
				newTrace.add(newEvent);
			}
			newLog.add(newTrace);
			context.getProgress().inc();
		}
		return newLog;
	}

	private String getName(XEvent event) {
		if (event.getAttributes().get("concept:name") == null)
			return "";
		return ((XAttributeLiteralImpl) event.getAttributes().get("concept:name")).getValue();
	}

	private void setName(XEvent event, String name) {
		event.getAttributes().put("concept:name", new XAttributeLiteralImpl("concept:name", name));
	}

	private boolean isCompleteEvent(XEvent event) {
		if (event.getAttributes().get("lifecycle:transition") == null)
			return true;
		return ((XAttributeLiteralImpl) event.getAttributes().get("lifecycle:transition")).getValue()
				.equals("complete");
	}

	private static XAttributeMap copyAttMap(XAttributeMap srcAttMap, RenameMergeEventSettings settings) {
		XAttributeMap destAttMap = new XAttributeMapImpl();
		Iterator<XAttribute> attit = srcAttMap.values().iterator();
		while (attit.hasNext()) {
			XAttribute att = attit.next();
			String key = att.getKey();
			att = (XAttribute) att.clone();

			if (key.equals("concept:name")) {
				for (int i = 0; i < settings.numRenameSettings("Task"); i++)
					if (((XAttributeLiteralImpl) att).getValue().equals(settings.getRenameSetting("Task", i).getFrom()))
						((XAttributeLiteralImpl) att).setValue(settings.getRenameSetting("Task", i).getInto());
			} else if (key.equals("org:resource")) {
				for (int i = 0; i < settings.numRenameSettings("Originator"); i++)
					if (((XAttributeLiteralImpl) att).getValue().equals(
							settings.getRenameSetting("Originator", i).getFrom()))
						((XAttributeLiteralImpl) att).setValue(settings.getRenameSetting("Originator", i).getInto());
			}
			destAttMap.put(key, att);
		}
		return destAttMap;
	}
}
