package org.processmining.plugins.simplelogoperations;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class RenameMergeEventsPanel extends JPanel {
	private static final long serialVersionUID = 0;

	private JPanel renamePanel, mergePanel;
	private JLabel renameTitle, mergeTitle;
	private JPanel renameScroll, mergeScroll;
	private ArrayList<RenamePanel> renamePanels;
	private ArrayList<MergePanel> mergePanels;
	private JButton bAddRenamePanel, bAddMergePanel;

	public RenameMergeEventsPanel() {
		SlickerFactory factory = SlickerFactory.instance();
//		SlickerDecorator decorator = SlickerDecorator.instance();

		renamePanel = factory.createRoundedPanel(15, Color.gray);

		renameTitle = factory.createLabel("Rename events");
		renameTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		renameTitle.setForeground(new Color(40, 40, 40));

		renameScroll = new JPanel();
		renameScroll.setBackground(Color.gray);
		renameScroll.setLayout(null);
		JScrollPane rsp = new JScrollPane(renameScroll);
		rsp.setBackground(Color.gray);
		renamePanels = new ArrayList<RenamePanel>();
		renamePanels.add(newRenamePanel());
		bAddRenamePanel = factory.createButton("Add");
		bAddRenamePanel.setForeground(new Color(40, 40, 40));
		bAddRenamePanel.addActionListener(new AddActionListener());

		mergePanel = factory.createRoundedPanel(15, Color.gray);

		mergeTitle = factory.createLabel("Merge consecutive events");
		mergeTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		mergeTitle.setForeground(new Color(40, 40, 40));

		mergeScroll = new JPanel();
		mergeScroll.setBackground(Color.gray);
		mergeScroll.setLayout(null);
		JScrollPane gsp = new JScrollPane(mergeScroll);
		gsp.setBackground(Color.gray);
		mergePanels = new ArrayList<MergePanel>();
		mergePanels.add(newMergePanel());
		bAddMergePanel = factory.createButton("Add");
		bAddMergePanel.setForeground(new Color(40, 40, 40));
		bAddMergePanel.addActionListener(new AddActionListener());

		renamePanel.setLayout(null);
		renamePanel.add(renameTitle);
		renamePanel.add(rsp);
		renamePanel.add(bAddRenamePanel);

		mergePanel.setLayout(null);
		mergePanel.add(mergeTitle);
		mergePanel.add(gsp);
		mergePanel.add(bAddMergePanel);

		renamePanel.setBounds(0, 0, 750, 200);
		renameTitle.setBounds(10, 10, 500, 30);
		rsp.setBounds(20, 50, 700, 120);
		bAddRenamePanel.setBounds(20, 175, 250, 20);

		mergePanel.setBounds(0, 210, 750, 200);
		mergeTitle.setBounds(10, 10, 500, 30);
		gsp.setBounds(20, 50, 700, 120);
		bAddMergePanel.setBounds(20, 175, 250, 20);

		setLayout(null);
		add(renamePanel);
		add(mergePanel);
		refresh();
	}

	private void refresh() {
		renameScroll.removeAll();
		for (int i = 0; i < renamePanels.size(); i++) {
			RenamePanel rp = renamePanels.get(i);
			rp.setFirst(false);
			rp.setLast(false);
			rp.setBounds(2, 2 + i * (RenamePanel.HEIGHT + 2), RenamePanel.WIDTH, RenamePanel.HEIGHT);
			renameScroll.add(rp);
		}
		if (renamePanels.size() > 0) {
			renamePanels.get(0).setFirst(true);
			renamePanels.get(renamePanels.size() - 1).setLast(true);
		}
		renameScroll.setPreferredSize(new Dimension(4 + RenamePanel.WIDTH, 2 + renamePanels.size()
				* (RenamePanel.HEIGHT + 2)));
		renameScroll.revalidate();

		mergeScroll.removeAll();
		for (int i = 0; i < mergePanels.size(); i++) {
			MergePanel mp = mergePanels.get(i);
			mp.setFirst(false);
			mp.setLast(false);
			mp.setBounds(2, 2 + i * (MergePanel.HEIGHT), MergePanel.WIDTH, MergePanel.HEIGHT);
			mergeScroll.add(mp);
		}
		if (mergePanels.size() > 0) {
			mergePanels.get(0).setFirst(true);
			mergePanels.get(mergePanels.size() - 1).setLast(true);
		}
		mergeScroll.setPreferredSize(new Dimension(2 + MergePanel.WIDTH, 2 + mergePanels.size()
				* (MergePanel.HEIGHT + 2)));
		mergeScroll.revalidate();

		validate();
		repaint();
	}

	public RenameMergeEventSettings getSettings() {
		RenameMergeEventSettings settings = new RenameMergeEventSettings();
		for (int i = 0; i < renamePanels.size(); i++) {
			RenamePanel rp = renamePanels.get(i);
			if (!rp.getFrom().equals("from") || !rp.getInto().equals("into"))
				settings.addRenameSetting(new RenameSetting(rp.getField(), rp.getFrom(), rp.getInto()));
		}
		for (int i = 0; i < mergePanels.size(); i++) {
			MergePanel mp = mergePanels.get(i);
			if (!mp.getFrom1().equals("from") || !mp.getFrom2().equals("from") || !mp.getInto().equals("into"))
				settings.addMergeSetting(new MergeSetting(mp.getFrom1(), mp.getFrom2(), mp.getInto()));
		}
		return settings;
	}

	private RenamePanel newRenamePanel() {
		RenamePanel rp = new RenamePanel(true, true);
		rp.upButton().addActionListener(new UpActionListener());
		rp.downButton().addActionListener(new DownActionListener());
		rp.removeButton().addActionListener(new RemoveActionListener());
		return rp;
	}

	private MergePanel newMergePanel() {
		MergePanel mp = new MergePanel(true, true);
		mp.upButton().addActionListener(new UpActionListener());
		mp.downButton().addActionListener(new DownActionListener());
		mp.removeButton().addActionListener(new RemoveActionListener());
		return mp;
	}

	private class AddActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == bAddRenamePanel) {
				renamePanels.add(newRenamePanel());
				refresh();
			} else if (e.getSource() == bAddMergePanel) {
				mergePanels.add(newMergePanel());
				refresh();
			}
		}
	}

	private class RemoveActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JButton source = (JButton) e.getSource();
			if (source.getParent().getClass() == RenamePanel.class) {
				for (int i = 0; i < renamePanels.size(); i++) {
					if (renamePanels.get(i) == source.getParent()) {
						renamePanels.remove(i);
						refresh();
						break;
					}
				}
			} else if (source.getParent().getClass() == MergePanel.class) {
				for (int i = 0; i < mergePanels.size(); i++) {
					if (mergePanels.get(i) == source.getParent()) {
						mergePanels.remove(i);
						refresh();
						break;
					}
				}
			}
		}
	}

	private class UpActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JButton source = (JButton) e.getSource();
			if (source.getParent().getClass() == RenamePanel.class) {
				for (int i = renamePanels.size() - 1; i > 0; i--) {
					if (renamePanels.get(i) == source.getParent()) {
						RenamePanel temp = renamePanels.get(i);
						renamePanels.set(i, renamePanels.get(i - 1));
						renamePanels.set(i - 1, temp);
						refresh();
						break;
					}
				}
			} else if (source.getParent().getClass() == MergePanel.class) {
				for (int i = mergePanels.size() - 1; i > 0; i--) {
					if (mergePanels.get(i) == source.getParent()) {
						MergePanel temp = mergePanels.get(i);
						mergePanels.set(i, mergePanels.get(i - 1));
						mergePanels.set(i - 1, temp);
						refresh();
						break;
					}
				}
			}
		}
	}

	private class DownActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JButton source = (JButton) e.getSource();
			if (source.getParent().getClass() == RenamePanel.class) {
				for (int i = 0; i < renamePanels.size() - 1; i++) {
					if (renamePanels.get(i) == source.getParent()) {
						RenamePanel temp = renamePanels.get(i);
						renamePanels.set(i, renamePanels.get(i + 1));
						renamePanels.set(i + 1, temp);
						refresh();
						break;
					}
				}
			} else if (source.getParent().getClass() == MergePanel.class) {
				for (int i = 0; i < mergePanels.size() - 1; i++) {
					if (mergePanels.get(i) == source.getParent()) {
						MergePanel temp = mergePanels.get(i);
						mergePanels.set(i, mergePanels.get(i + 1));
						mergePanels.set(i + 1, temp);
						refresh();
						break;
					}
				}
			}
		}
	}
}
